; SketchyLISP Extension Library
; Copyright (C) 2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; for-all

; ---conformance---
; R5.92RS

; ---purpose---
; Test whether all members of a sequence of lists have a
; given property.
; The property is expressed using an
; .V n-ary
; function
; .V p.
; .V P
; is applied to a list consisting of the first
; member of each given list. Hence there must be
; exactly
; .V n
; lists. If
; .V p
; returns truth, it is applied to a list consisting
; of the second member of each given list, etc.
; If
; .V p
; returns falsity for any set of members,
; .C for-all
; returns
; .C #f.
; If only one set of members is left to check,
; .C for-all
; returns the value of
; .V p
; applied to this last set.
; .B
; All lists passed to
; .C for-all
; must have the same length.
; .B
; When all lists passed to
; .C for-all
; are empty, it returns
; .C #t.

; ---args---
; P - predicate
; A* ... - lists

; ---keywords---
; FORALL function, universal quantifier, property, predicate

; ---see-also---
; exists, memp, filter

; ---example---
; (for-all < '(5 7 3) '(6 8 4)) => #t

; (require "nullp.scm") ; null?
; (require "lib/map-car.scm")

; ---code---
(define (for-all p . a*)
  (letrec
    ((carof
       (lambda (a)
         (map-car car a)))
     (cdrof
       (lambda (a)
         (map-car cdr a)))
     (forall
       (lambda (a*)
         (cond ((null? (car a*)) #t)
           ((null? (cdar a*))
             (apply p (carof a*)))
           (else (and (apply p (carof a*))
                      (forall (cdrof a*))))))))
    (forall a*)))
