; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; intersection

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Compute the intersection of sets.
; Sets are represented by lists of unique
; members.

; ---args---
; A*... - sets

; ---keywords---
; INTERSECTION function, intersection
; sets, set theory

; ---see-also---
; union, unique, member

; ---example---
; (intersection '(a b c) '(b c d) '(c d e)) => (c)

; (require "nullp.scm") ; null?
; (require "lib/fold-left.scm")
; (require "lib/reverse.scm")
; (require "lib/member.scm")

; ---code---
(define (intersection . a*)
  (letrec
    ((_intersection
       (lambda (a b)
         (cond ((null? a) '())
           ((member (car a) b)
             (cons (car a) (_intersection (cdr a) b)))
           (else (_intersection (cdr a) b))))))
    (cond ((null? a*) '())
      (else (fold-left _intersection (car a*) (cdr a*))))))
