; SketchyLISP Extension Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; permute

; ---conformance---
; SketchyLISP Extension

; ---purpose---
; Generate permutations of a set.

; ---args---
; X - set

; ---keywords---
; PERMUTE, permutation, combination

; ---see-also---
; iota, factorial

; ---example---
; (permute '(a b c)) => ((a b c) (a c b) (b c a) (b a c) (c a b) (c b a))

; (require "nullp.scm") ; null?
; (require "lib/list.scm")
; (require "lib/append.scm")
; (require "lib/map.scm")

; ---code---
(define (permute x)
  (letrec
    ((rotate
       (lambda (x)
         (append (cdr x) (list (car x)))))
     (rotations
       (lambda (x)
         (letrec
           ((rot (lambda (x n)
             (cond ((null? n) '())
               (else (cons x (rot (rotate x)
                                  (cdr n))))))))
           (rot x x))))
     (permutations
       (lambda (lst)
         (cond ((null? lst) '())
           ((null? (cdr lst)) (list lst))
           ((null? (cddr lst)) (rotations lst))
           (else (apply append
                   (map (lambda (rotn)
                          (map (lambda (x)
                                 (cons (car rotn) x))
                               (permutations (cdr rotn))))
                        (rotations lst))))))))
    (permutations x)))
