; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; assv

; ---conformance---
; R5RS

; ---purpose---
; Retrieve an association from an association list.
; An association list is a list of pairs where
; the
; .E car
; part of each pair holds a key and the
; .E cdr
; part of the pair holds the value
; associated with that key:
; .B
; .C ((key1 . value1) ... (keyN . valueN))
; .B
; .C Assv
; returns the first association whose key is equivalent
; to a given atom.

; ---args---
; X - key of value to be found
; A - association list

; ---keywords---
; ASSV function, association lists, alists, associate

; ---see-also---
; assoc, assq, memv

; ---example---
; (assv '2 '((1 . i) (2 . ii) (3 . iii) (4 . iv))) => (2 . ii)

(define assv #t)

; (require "nullp.scm") ; null?
(require "caar.scm") ; cdar
(require "eqvp.scm") ; eqv?

; ---code---
(define (assv x a)
  (cond ((null? a) #f)
    ((eqv? (caar a) x) (car a))
    (else (assv x (cdr a)))))
