; SketchyLISP Library
; Copyright (C) 2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; fold-left

; ---conformance---
; R5.91RS

; ---purpose---
; Iterate over lists. Combine the base element
; .V b
; with the list of the first members of each given list.
; Combine the result with the list of their second
; members, etc.
; When the given lists are empty, return the base
; element.
; .B
; All lists passed to
; .C fold-left
; must have the same length.

; ---args---
; F - function to apply
; B - base element
; A* ... - lists

; ---keywords---
; FOLD-LEFT function, folding, lists, left-associative

; ---see-also---
; fold-right, map, member, list?

; ---example---
; :l lib/list.scm
; (fold-left list '1 '(a b c) '(d e f)) => (((1 a d) b e) c f)

(define fold-left #t)

; (require "nullp.scm") ; null?
(require "map-car.scm")

; ---code---
(define (fold-left f b . a*)
  (letrec
    ((carof
       (lambda (a)
         (map-car car a)))
     (cdrof
       (lambda (a)
         (map-car cdr a)))
     (fold
       (lambda (a* r)
         (cond ((null? (car a*)) r)
           (else (fold (cdrof a*)
                   (apply f r (carof a*))))))))
    (cond ((null? a*)
        (bottom '(too few arguments to fold-left)))
      ((null? (car a*)) b)
      (else (fold a* b)))))
