; SketchyLISP Library
; Copyright (C) 2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; fold-right

; ---conformance---
; R5.91RS

; ---purpose---
; Iterate over lists right-associatively. Combine the list
; of last members of the given lists with the base element
; .V b.
; Combine the result with the list of second-to-last
; members, etc.
; When the given lists are empty, return the base
; element.
; .B
; All lists passed to
; .C fold-right
; must have the same length.

; ---args---
; F - function to apply
; B - base element
; A* ... - lists

; ---keywords---
; FOLD-RIGHT function, folding, lists, right-associative

; ---see-also---
; fold-left, map, member, list?

; ---example---
; :l lib/list.scm
; (fold-right list '9 '(a b c) '(d e f)) => (a d (b e (c f 9)))

(define fold-right #t)

; (require "nullp.scm") ; null?
(require "map-car.scm")
(require "append.scm")
(require "reverse.scm")
(require "map.scm")

; ---code---
(define (fold-right f b . a*)
  (letrec
    ((carof
       (lambda (a)
         (map-car car a)))
     (cdrof
       (lambda (a)
         (map-car cdr a)))
     (foldr
       (lambda (a* r)
         (cond ((null? (car a*)) r)
           (else (foldr (cdrof a*)
                   (apply f (append (carof a*) (list r)))))))))
    (cond ((null? a*)
        (bottom '(too few arguments to fold-right)))
      ((null? (car a*)) b)
      (else (foldr (map reverse a*) b)))))
