; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; gcd

; ---conformance---
; R5RS

; ---purpose---
; Compute the greatest common divisor of a sequence of numbers.
; This function uses Euclid's Algorithm.

; ---args---
; A... - numbers

; ---keywords---
; GCD function, gcd, greatest common divisor
; Euclid, Euclid's algorithm, arithmetics

; ---see-also---
; digits, expt, lcm

; ---example---
; (gcd 99 -77) => 11

(define gcd #t)

(require "abs.scm")
(require "natural.scm")
(require "zerop.scm") ; zero?
(require "nremainder.scm")
(require "map.scm")
(require "fold-left.scm")

; ---code---
(define (gcd . a)
  (letrec
    ((_gcd
       (lambda (a b)
         (cond ((zero? b) a)
           ((zero? a) b)
           ((n< a b) (_gcd a (nremainder b a)))
           (else (_gcd b (nremainder a b)))))))
    (fold-left _gcd 0 (map (lambda (x)
                             (natural (abs x)))
                           a))))
