; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; n*

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Compute the product of two natural numbers.

; ---args---
; A - natural number
; B - natural number

; ---keywords---
; N* function, times, multiply, multiplication, product
; arithmetics, natural

; ---see-also---
; digits, nquotient, nremainder, n+, n-, *

; ---example---
; (n* 4 5) => 20

(define ntimes #t)

; (require "nullp.scm") ; null?
(require "digits.scm") ; 0d
(require "list.scm")
(require "append.scm")
(require "reverse.scm")
(require "zerop.scm") ; zero?

; ---code---
(define (n* a b)
  (letrec
    ; X*10 where X=/=0
    ((x10
       (lambda (x)
         (list->integer
           (append (integer->list x) '(0d))
           #t)))
     ; Add A to R B times.
     ; A,R are numbers, B is a decimal digit.
     (addn
       (lambda (a b r)
         (cond ((zero? b) r)
           (else (addn a (n- b 1) (n+ a r))))))
     ; R=A*B
     ; B is in reverse order.
     (tms
       (lambda (a b r)
         (cond ((null? b) r)
           (else (tms (x10 a) (cdr b)
                      (addn a (list->integer (list (car b)))
                            r)))))))
    ; avoid leading zeroes in result
    (cond ((zero? a) 0)
      (else (tms a (reverse (integer->list b))
              0)))))
