; SketchyLISP Library
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; sqrt

; ---conformance---
; R5RS
; Result is an integer.

; ---purpose---
; Compute the square root of a natural number.
; .E (In fact, this function computes the greatest
; .E integer that is no greater than the square
; .E root of the given argument.)

; ---args---
; X - square of root to extract

; ---keywords---
; SQRT function, root, square, square root, arithmetics

; ---see-also---
; digits, expt, gcd

; ---example---
; (sqrt 144) => 12

(define sqrt #t)

(require "list.scm")
(require "equal.scm") ; =
(require "negativep.scm") ; negative?
(require "ntimes.scm") ; n*
(require "nquotient.scm")
(require "ngreater.scm") ; n>
(require "nlteq.scm") ; n<=

; ---code---
(define (sqrt square)
  (letrec
    ((_sqrt (lambda (x last)
       (cond ((= last x) x)
         ((= last (n+ 1 x))
           (if (n> (n* x x) square) (- x 1) x))
         (else (_sqrt (nquotient
                        (n+ x (nquotient square x))
                        2)
                      x))))))
    (cond ((negative? square)
        (bottom (list 'sqrt square)))
      (else (_sqrt square 0)))))
