; SketchyLISP Example Program
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; lvrename

; ---language---
; R5RS Scheme + SketchyLISP Ext.

; ---purpose---
; Rename variables of lambda expressions.
; Lambda variables are renamed in such a way
; that no conflicts can arise between names
; of nested lambda expressions. After
; lvrename-ing an expression, full beta
; reduction can be done using
; .C subst.
; .P
; For instance,
; .B
; .C (lvrename '(lambda (x) (lambda (x) x)))
; .B
; .C => (lambda (x:) (lambda (x:i) x:i))

; ---args---
; X - S-expression

; ---keywords---
; LVRENAME function, lambda abstraction, beta reduction
; lambda variables, lambda variable renaming, example

; ---example---
; (lvrename '(lambda (x) (lambda (x) x))) => (lambda (x:) (lambda (x:i) x:i))

; (require "lib/list.scm")
; (require "lib/assq.scm")
; (require "lib/append.scm")
; (require "lib/map.scm")
; (require "lib/caar.scm") ; cadr
; (require "lib/caaar.scm") ; caddr

; ---code---
(require "explode.scm")
(require "implode.scm")

(define (lvrename expr)
  (letrec
    ((add
       (lambda (name level)
         (implode (append (explode name) '(:) level))))
     (ext-sub
       (lambda (sub vars level)
         (letrec
           ((add-var
              (lambda (name level alist)
                (cons (cons name (add name level)) alist))))
           (cond ((null? vars) sub)
             ((symbol? vars) (add-var vars level sub))
             (else (ext-sub (add-var (car vars) level sub)
                            (cdr vars)
                            level))))))
     (subst
       (lambda (name sub)
         (let ((v (assq name sub)))
           (if v (cdr v) name))))
     (rename-vars
       (lambda (expr sub level)
         (cond ((symbol? expr) (subst expr sub))
           ((not (pair? expr)) expr)
           ((eq? (car expr) 'quote) expr)
           ((eq? (car expr) 'lambda)
             (let* ((args (cadr expr))
                    (body (caddr expr))
                    (new-sub (ext-sub sub args level)))
               (list 'lambda
                      (rename-vars args new-sub level)
                      (rename-vars body new-sub
                                   (cons 'i level)))))
           (else (map (lambda (x) (rename-vars x sub level))
                      expr))))))
    (rename-vars expr '() '())))
