; SketchyLISP Example Program
; Copyright (C) 2005,2006,2007 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; unsort

; ---language---
; R5RS Scheme

; ---purpose---
; Generate some entropy by un-sorting a list of
; natural numbers. The order of the list
; .V a
; is changed in a hard to predict way by extracting
; the
; .V (n mod length[a])'th
; element from
; .V a
; and consing it to
; .V b.
; Each extracted element is removed from
; .V a.
; When
; .V a
; is empty, the result is in
; .V b.

; ---args---
; A - list to unsort
; SEED - first element to arrange, where 0<SEED<=length[A]

; ---keywords---
; UNSORT function, entropy, order, sorting, unsorting, random

; ---example---
; :l src/iota.scm
; (unsort (iota 1 5) 3) => (3 2 5 1 4)

; (require "lib/nullp.scm") ; null?
; (require "lib/digits.scm")
; (require "lib/length.scm")
; (require "lib/zerop.scm") ; zero?
; (require "lib/remainder.scm")
; (require "lib/minus.scm")
; (require "lib/list-ref.scm")

; ---code---
(define (unsort a seed)
  (letrec
    ; Remove n'th element.
    ((rem-nth
       (lambda (a n r)
         (cond ((zero? n) 
             (if (null? a)
                 (reverse r)
                 (append (cdr a) (reverse r))))
           (else (rem-nth (cdr a) (- n 1)
                          (cons (car a) r))))))
     ; Unsort list A of length K
     ; N = element to extract
     ; R = intermediate result
     (_unsort
       (lambda (a n k r)
         (cond ((zero? k) (cons (car a) r))
           (else (_unsort (rem-nth a n '())
                          (remainder (car a) k)
                          (- k 1)
                          (cons (list-ref a n) r)))))))
    (_unsort a seed (- (length a) 1) '())))
